@@ -15,17 +15,23 @@ my $external_data = {
 };
 
 my $schema = $ypp->schema;
-$schema->add_tag_resolver(
-    "!external" => regex => qr{^(.*)$} => sub {
-        my ($value) = @_;
+$schema->add_resolver(
+    tag => "!external",
+    match => [ regex => qr{^(.*)$} => sub {
+        my ($constructor, $event, $matches) = @_;
+        my ($value) = @$matches;
         path($external_data, $value)
-    }
+    }],
+    implicit => 0,
 );
-$schema->add_tag_resolver(
-    "!template" => regex => qr{^(.*)$} => sub {
-        my ($value) = @_;
+$schema->add_resolver(
+    tag => "!template",
+    match => [ regex => qr{^(.*)$} => sub {
+        my ($constructor, $event, $matches) = @_;
+        my ($value) = @$matches;
         template($external_data, $value)
-    }
+    }],
+    implicit => 0,
 );
 
 my $data = $ypp->load_file($filename);
