@@ -114,6 +114,35 @@ EOM
 - &1 !perl/scalar:Foo
   =: some other string
 - *1
+EOM
+    ],
+    refref => [
+        <<'EOM',
+        my $ref = { a => 'hash' };
+        my $refref = \$ref;
+        $refref;
+EOM
+        <<'EOM',
+---
+- &1 !perl/ref
+  =:
+    a: hash
+- *1
+EOM
+    ],
+
+    refref_blessed => [
+        <<'EOM',
+        my $ref = { a => 'hash' };
+        my $refref = bless \$ref, 'Foo';
+        $refref;
+EOM
+        <<'EOM',
+---
+- &1 !perl/ref:Foo
+  =:
+    a: hash
+- *1
 EOM
     ],
 );
