@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package YAML::PP::Emitter;
 
-our $VERSION = '0.010_003'; # TRIAL VERSION
+our $VERSION = '0.011_001'; # TRIAL VERSION
 
 use YAML::PP::Common qw/
     YAML_PLAIN_SCALAR_STYLE YAML_SINGLE_QUOTED_SCALAR_STYLE
@@ -269,7 +269,7 @@ sub sequence_end_event {
 
 my %forbidden_first = (qw/
     ! 1 & 1 * 1 { 1 } 1 [ 1 ] 1 | 1 > 1 @ 1 ` 1 " 1 ' 1
-/, '#' => 1, ',' => 1, " " => 1);
+/, '#' => 1, '%' => 1, ',' => 1, " " => 1);
 my %forbidden_first_plus_space = (qw/
     ? 1 - 1 : 1
 /);
