@@ -2,8 +2,9 @@ use strict;
 use warnings;
 package YAML::PP::Schema;
 use B;
+use Module::Load qw//;
 
-our $VERSION = '0.010_003'; # TRIAL VERSION
+our $VERSION = '0.011_001'; # TRIAL VERSION
 
 use YAML::PP::Common qw/ YAML_PLAIN_SCALAR_STYLE /;
 
@@ -58,7 +59,13 @@ sub bool_class { return $_[0]->{bool_class} }
 sub load_subschemas {
     my ($self, @schemas) = @_;
     for my $s (@schemas) {
-        my $class = "YAML::PP::Schema::" . $s;
+        my $class;
+        if ($s =~ m/^\:(.*)/) {
+          $class = "$1";
+          Module::Load::load $class;
+        } else {
+          $class = "YAML::PP::Schema::$s";
+        }
         my $tags = $class->register(
             schema => $self,
         );
@@ -163,19 +170,28 @@ sub add_representer {
     }
 }
 
-sub load_scalar_tag {
-    my ($self, $event) = @_;
+sub load_scalar {
+    my ($self, $constructor, $event) = @_;
     my $tag = $event->{tag};
     my $value = $event->{value};
-    $value = '' unless defined $value;
+
     my $resolvers = $self->resolvers;
-    my $res = $resolvers->{tag}->{ $tag };
+    my $res;
+    if ($tag) {
+        $res = $resolvers->{tag}->{ $tag };
+    }
+    else {
+        $res = $resolvers->{value};
+        if ($event->{style} ne YAML_PLAIN_SCALAR_STYLE) {
+            return $value;
+        }
+    }
 
     if (my $equals = $res->{equals}) {
         if (exists $equals->{ $value }) {
             my $res = $equals->{ $value };
             if (ref $res eq 'CODE') {
-                return $res->();
+                return $res->($constructor, $event);
             }
             return $res;
         }
@@ -185,39 +201,7 @@ sub load_scalar_tag {
             my ($re, $sub) = @$item;
             my @matches = $value =~ $re;
             if (@matches) {
-                return $sub->(@matches);
-            }
-        }
-        die "Tag $tag ($value)";
-    }
-#    die "Tag $tag ($value)";
-    return $value;
-}
-
-sub load_scalar {
-    my ($self, $style, $value) = @_;
-    if ($style ne YAML_PLAIN_SCALAR_STYLE) {
-        return $value;
-    }
-    my $resolvers = $self->resolvers;
-    my $res = $resolvers->{value};
-    $value = '' unless defined $value;
-
-    if (my $equals = $res->{equals}) {
-        if (exists $equals->{ $value }) {
-            my $res = $equals->{ $value };
-            if (ref $res eq 'CODE') {
-                return $res->();
-            }
-            return $res;
-        }
-    }
-    if (my $regex = $res->{regex}) {
-        for my $item (@$regex) {
-            my ($re, $sub) = @$item;
-            my @matches = $value =~ $re;
-            if (@matches) {
-                return $sub->(@matches);
+                return $sub->($constructor, $event, \@matches);
             }
         }
     }
@@ -276,10 +260,10 @@ sub new {
     return $self;
 }
 
-sub _to_int { 0 + $_[0] }
+sub _to_int { 0 + $_[2]->[0] }
 
 # DaTa++ && shmem++
-sub _to_float { unpack F => pack F => $_[0] }
+sub _to_float { unpack F => pack F => $_[2]->[0] }
 
 sub register {
     my ($self, %args) = @_;
@@ -312,7 +296,7 @@ sub register {
     );
     $schema->add_resolver(
         tag => 'tag:yaml.org,2002:str',
-        match => [ regex => qr{^(.*)$} => sub { $_[0] } ],
+        match => [ regex => qr{^(.*)$} => sub { $_[2]->[0] } ],
         implicit => 0,
     );
 
@@ -420,8 +404,8 @@ sub new {
     return $self;
 }
 
-sub _from_oct { oct $_[0] }
-sub _from_hex { hex $_[0] }
+sub _from_oct { oct $_[2]->[0] }
+sub _from_hex { hex $_[2]->[0] }
 
 sub register {
     my ($self, %args) = @_;
@@ -469,7 +453,7 @@ sub register {
     ) for (qw/ .nan .NaN .NAN /);
     $schema->add_resolver(
         tag => 'tag:yaml.org,2002:str',
-        match => [ regex => qr{^(.*)$} => sub { $_[0] } ],
+        match => [ regex => qr{^(.*)$} => sub { $_[2]->[0] } ],
         implicit => 0,
     );
 
